"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useCreateReducerWithEffect;
exports.SideEffect = exports.UpdateWithSideEffect = exports.NoUpdate = exports.Update = void 0;

var _react = require("react");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var NO_UPDATE_SYMBOL = Symbol("NO_UPDATE_SYMBOL");
var REMOVE_EXECUTED_SIDE_EFFECT = Symbol("REMOVE_EXECUTED_SIDE_EFFECT");

var Update = function Update(newState) {
  return {
    newState: newState
  };
};

exports.Update = Update;

var NoUpdate = function NoUpdate() {
  return NO_UPDATE_SYMBOL;
};

exports.NoUpdate = NoUpdate;

var UpdateWithSideEffect = function UpdateWithSideEffect(newState, newSideEffect) {
  return {
    newState: newState,
    newSideEffect: {
      effect: newSideEffect,
      id: Math.random() * 10000
    }
  };
};

exports.UpdateWithSideEffect = UpdateWithSideEffect;

var SideEffect = function SideEffect(newSideEffect) {
  return {
    newSideEffect: {
      effect: newSideEffect,
      id: Math.random() * 10000
    }
  };
};

exports.SideEffect = SideEffect;

function executeSideEffects(_x) {
  return _executeSideEffects.apply(this, arguments);
}

function _executeSideEffects() {
  _executeSideEffects = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(_ref) {
    var sideEffects, state, dispatch;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            sideEffects = _ref.sideEffects, state = _ref.state, dispatch = _ref.dispatch;
            _context2.next = 3;
            return Promise.all(sideEffects.forEach(function (sideEffect) {
              sideEffect.effect(state, dispatch);
              dispatch({
                type: REMOVE_EXECUTED_SIDE_EFFECT,
                sideEffect: sideEffect
              });
            }));

          case 3:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _executeSideEffects.apply(this, arguments);
}

function finalReducer(reducer) {
  return function (_ref2, action) {
    var state = _ref2.state,
        _ref2$sideEffects = _ref2.sideEffects,
        sideEffects = _ref2$sideEffects === void 0 ? [] : _ref2$sideEffects;

    var _reducer = reducer(state, action),
        newState = _reducer.newState,
        newSideEffect = _reducer.newSideEffect;

    var newSideEffects = sideEffects;
    newSideEffects = newSideEffect ? [].concat(_toConsumableArray(newSideEffects), [newSideEffect]) : newSideEffects;

    if (action.type === REMOVE_EXECUTED_SIDE_EFFECT) {
      newSideEffects = sideEffects.filter(function (effect) {
        return effect.id !== action.sideEffect.id;
      });
    } else if (action === NO_UPDATE_SYMBOL) {
      return {
        state: state,
        sideEffects: sideEffects
      };
    }

    return {
      state: newState || state,
      sideEffects: newSideEffects
    };
  };
}

function useCreateReducerWithEffect(reducer, initialState) {
  var _useReducer = (0, _react.useReducer)(finalReducer(reducer), {
    state: initialState,
    sideEffects: []
  }),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      _useReducer2$ = _useReducer2[0],
      state = _useReducer2$.state,
      sideEffects = _useReducer2$.sideEffects,
      dispatch = _useReducer2[1];

  (0, _react.useEffect)(function () {
    if (sideEffects.length) {
      var runSideEffects =
      /*#__PURE__*/
      function () {
        var _ref3 = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee() {
          return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  _context.next = 2;
                  return executeSideEffects({
                    sideEffects: sideEffects,
                    state: state,
                    dispatch: dispatch
                  });

                case 2:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));

        return function runSideEffects() {
          return _ref3.apply(this, arguments);
        };
      }();

      runSideEffects();
    }
  }, [sideEffects]); //eslint-disable-line

  return [state, dispatch];
}